/*
 *  Sshtools - Java SSH2 API
 *
 *  Copyright (C) 2002 Lee David Painter.
 *
 *  Written by: 2002 Lee David Painter <lee@sshtools.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package com.sshtools.j2ssh.sftp;

import com.sshtools.j2ssh.io.ByteArrayReader;
import com.sshtools.j2ssh.io.ByteArrayWriter;
import com.sshtools.j2ssh.io.UnsignedInteger32;

import com.sshtools.j2ssh.subsystem.SubsystemMessage;


public class SshFxpHandle extends SubsystemMessage implements MessageRequestId {
    public static final int SSH_FXP_HANDLE = 102;
    private UnsignedInteger32 id;
    private byte[] handle;

    public SshFxpHandle(UnsignedInteger32 id, byte[] handle) {
        super(SSH_FXP_HANDLE);
        this.id = id;
        this.handle = handle;
    }

    public SshFxpHandle() {
        super(SSH_FXP_HANDLE);
    }

    public UnsignedInteger32 getId() {
        return id;
    }

    public byte[] getHandle() {
        return handle;
    }

    public void constructMessage(ByteArrayReader bar)
        throws java.io.IOException, 
            com.sshtools.j2ssh.transport.InvalidMessageException {
        id = bar.readUINT32();
        handle = bar.readBinaryString();
    }

    public String getMessageName() {
        return "SSH_FXP_HANDLE";
    }

    public void constructByteArray(ByteArrayWriter baw)
        throws java.io.IOException, 
            com.sshtools.j2ssh.transport.InvalidMessageException {
        baw.writeUINT32(id);
        baw.writeBinaryString(handle);
    }
}
