/*
 *  Sshtools - Java SSH2 API
 *
 *  Copyright (C) 2002 Lee David Painter.
 *
 *  Written by: 2002 Lee David Painter <lee@sshtools.com>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package com.sshtools.j2ssh.transport.kex;

import com.sshtools.j2ssh.io.ByteArrayReader;
import com.sshtools.j2ssh.io.ByteArrayWriter;

import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;

import java.io.IOException;

import java.math.BigInteger;


public class SshMsgKexDhInit extends SshMessage {
    protected final static int SSH_MSG_KEXDH_INIT = 30;

    // Stores the e value
    private BigInteger e;

    public SshMsgKexDhInit(BigInteger e) {
        super(SSH_MSG_KEXDH_INIT);
        this.e = e;
    }

    public SshMsgKexDhInit() {
        super(SSH_MSG_KEXDH_INIT);
    }

    public BigInteger getE() {
        return e;
    }

    public String getMessageName() {
        return "SSH_MSG_KEXDH_INIT";
    }

    protected void constructByteArray(ByteArrayWriter baw)
        throws InvalidMessageException {
        try {
            baw.writeBigInteger(e);
        } catch (IOException ioe) {
            throw new InvalidMessageException("Error writing message data: "
                + ioe.getMessage());
        }
    }

    protected void constructMessage(ByteArrayReader bar)
        throws InvalidMessageException {
        try {
            e = bar.readBigInteger();
        } catch (IOException ioe) {
            throw new InvalidMessageException("Error reading message data: "
                + ioe.getMessage());
        }
    }
}
